//+------------------------------------------------------------------+
//|                                                      Leader MACD |
//|                                                   LeaderMACD.mq4 |
//|                                                           mladen |
//|                                                                  |
//| TASC jul 2008                                                    |
//| Giorgos Siligardos                                               |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DeepSkyBlue
#property indicator_color2  OrangeRed
#property indicator_color3  Goldenrod
#property indicator_color4  Gray
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame      = "current time frame";
extern int    FastEMA        = 12;
extern int    SlowEMA        = 26;
extern int    SignalEMA      =  9;
extern int    Price          = PRICE_CLOSE;
extern bool   SignalOnLeader = false;
extern bool   ShowMACD       = true;
extern bool   ShowSignal     = true;
extern bool   Interpolate    = true;

//
//
//
//
//

double macd[];
double leader[];
double signal[];
double osma[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   if (!ShowMACD) SignalOnLeader = true;
   
   //
   //
   //
   //
   //
      
      SetIndexBuffer(0,macd);
         if (ShowMACD) 
               SetIndexStyle(0,DRAW_LINE);
         else  SetIndexStyle(0,DRAW_NONE);
      SetIndexBuffer(1,leader);
      SetIndexBuffer(2,signal);
      SetIndexBuffer(3,osma);   SetIndexStyle(3,DRAW_HISTOGRAM);
   
   //
   //
   //
   //
   //
   
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="CalculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   //
   //
   //
   //
   //
      
   IndicatorShortName(timeFrameToString(timeFrame)+" Leader MACD ("+FastEMA+","+SlowEMA+","+SignalEMA+")");
   return(0);
}
int deinit() { return(0); }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double tBuffer[][5];
#define iFastEma 0
#define iSlowEma 1
#define iFastEmx 2
#define iSlowEmx 3
#define iSmoothl 4

int start()
{
   int i,r,limit;
   int counted_bars=IndicatorCounted();
   if (counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { macd[0] = limit+1; return(0); }
 
   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);
      double alphaFast = 2.0 / (1.0+FastEMA);
      double alphaSlow = 2.0 / (1.0+SlowEMA);
      double alphaSign = 2.0 / (1.0+SignalEMA);

      //
      //
      //
      //
      //
         
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      
         tBuffer[r][iFastEma] = iMA(NULL,0,FastEMA,0,MODE_EMA,Price,i);
         tBuffer[r][iSlowEma] = iMA(NULL,0,SlowEMA,0,MODE_EMA,Price,i);
         tBuffer[r][iFastEmx] = tBuffer[r-1][iFastEmx] + alphaFast*(price-tBuffer[r][iFastEma]-tBuffer[r-1][iFastEmx]);
         tBuffer[r][iSlowEmx] = tBuffer[r-1][iSlowEmx] + alphaSlow*(price-tBuffer[r][iSlowEma]-tBuffer[r-1][iSlowEmx]);
         
         //
         //
         //
         //
         //
         
         leader[i] = tBuffer[r][iFastEma] + tBuffer[r][iFastEmx]   - tBuffer[r][iSlowEma] - tBuffer[r][iSlowEmx];
                     tBuffer[r][iSmoothl] = tBuffer[r-1][iSmoothl] + alphaSign*(leader[i] - tBuffer[r-1][iSmoothl]);
 
         //
         //
         //
         //
         //
                             
         macd[i] = tBuffer[r][iFastEma]-tBuffer[r][iSlowEma];
         if (ShowSignal)                                   
         if (SignalOnLeader)                     
            {
               signal[i] = leader[i]-tBuffer[r][iSmoothl];
               osma[i]   = leader[i]-signal[i];
            }               
         else
            {
               signal[i] = signal[i+1] + alphaSign*(macd[i]-signal[i+1]);
               osma[i]   = macd[i]-signal[i];
            }               
      }                  
      return(0);
   }      
   
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         macd[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,SignalEMA,Price,SignalOnLeader,ShowMACD,ShowSignal,0,y);
         leader[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,SignalEMA,Price,SignalOnLeader,ShowMACD,ShowSignal,1,y);
         signal[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,SignalEMA,Price,SignalOnLeader,ShowMACD,ShowSignal,2,y);
         osma[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",FastEMA,SlowEMA,SignalEMA,Price,SignalOnLeader,ShowMACD,ShowSignal,3,y);

         //
         //
         //
         //
         //
      
            if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
            if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               macd[i+k]   = macd[i]   + (macd[i+n]   - macd[i]  )*k/n;
               leader[i+k] = leader[i] + (leader[i+n] - leader[i])*k/n;
               osma[i+k]   = osma[i]   + (osma[i+n]   - osma[i]  )*k/n;
               signal[i+k] = signal[i] + (signal[i+n] - signal[i])*k/n;
            }               
   }
   
   //
   //
   //
   //
   //
      
   return(0);   
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}